@@ -111,7 +111,7 @@ gcmHash_InitContext(gcmHashContext *ghash, const unsigned char *H, PRBool sw)
         rv = gcm_HashInit_hw(ghash);
     } else {
 /* We fall back to the software implementation if we can't use / don't
-         * want to use pclmul. */
+ * want to use pclmul. */
 #ifdef HAVE_INT128_SUPPORT
         ghash->ghash_mul = gcm_HashMult_sftw;
 #else
@@ -491,9 +491,12 @@ gcmHash_Reset(gcmHashContext *ghash, const unsigned char *AAD,
     SECStatus rv;
 
     // Limit AADLen in accordance with SP800-38D
-    if (sizeof(AADLen) >= 8 && AADLen > (1ULL << 61) - 1) {
-        PORT_SetError(SEC_ERROR_INPUT_LEN);
-        return SECFailure;
+    if (sizeof(AADLen) >= 8) {
+        unsigned long long AADLen_ull = AADLen;
+        if (AADLen_ull > (1ULL << 61) - 1) {
+            PORT_SetError(SEC_ERROR_INPUT_LEN);
+            return SECFailure;
+        }
     }
 
     ghash->cLen = 0;
